# Tests Documentation

Install pytest & pytest-qt:

```bash
cd hev
. .hev_env/bin/activate
pip3 install pytest
pip3 install pytest-qt
```

⚠️ For unit and integration testing, remember️ to export your PYTHONPATH environment variable as shown below.

## Unit Tests

To run all unit tests in the `NativeUI/tests/unittests` dir on a Raspberry Pi or VM, run the following 
(adjust the full path to your NativeUI directory accordingly):

```bash
cd hev
. .hev_env/bin/activate
export PYTHONPATH=/home/pi/hev/NativeUI
pytest NativeUI/tests/unittests
```

To run a single unit test file, set the environment as above and specify the test file:

```bash
pytest NativeUI/tests/unittests/test_hevclient.py
```

## Integration Tests

* ⚠️ Integration tests automatically start / stop the ArduinoEmulator & hevserver, there is no longer any need to run
them manually from the CLI.
* ⚠️ Some integration tests spin up the GUI and simulate button clicks / and interactions with the GUI. 
These tests then assert that the state of the GUI has changed as expected. 
Therefore, a DISPLAY is (currently) needed.

To run all integration tests in the `NativeUI/tests/integration` dir:

```bash
cd hev
. .hev_env/bin/activate
export PYTHONPATH=/home/pi/hev/NativeUI
pytest NativeUI/tests/integration
```

* To run a single integration test file, set the environment as above and specify the test file:

```bash
pytest NativeUI/tests/integration/test_hevclient_integration.py
```


### Integration tests automatically start / stop the ArduinoEmulator & hevserver 

`integration_utils.py` has functions to start/manage/kill the ArduinoEmulator and hevserver
from within pytest code INSTEAD of running these background processes separately on the CLI.
Nothing wrong with using the CLI, but managing these processes from within test-code itself
can be useful - tests can be more atomic and easier to manage: e.g. you can choose to
start/kill these background processes for each/every test individually
and/or once for a particular test-suite.

The `sample_test_hevclient_integration.py` shows sample usage (note, don't run
this as a pytest if hevserver/ArduinoEmulator is already running, it will clash).
This sample integration test is purposefully not pre/appended with 'test' so that
it is excluded from pytest by default, but you can run it explicitly as below:

```bash
cd hev
. .hev_env/bin/activate
export PYTHONPATH=/home/pi/hev/NativeUI
pytest NativeUI/tests/integration/sample_test_hevclient_integration.py
```

## Coverage

To get pytest coverage run from the root of the repo:

```bash
pip install pytest-cov
pytest --cov=NativeUI NativeUI
```


## System Tests

### Template

Status is marked in the test title with:
* :x: for not started
* :large_orange_diamond: for WIP
* :white_check_mark: for completed

RiskID | Domain | Functional Area | Standard Reference | Assignee
------ | ------ | --------------- | ------------------ | --------
SW | Software-GUI | Alarms | ISO-XX | Tim Powell

#### Scenario: <EXAMPLE>

    GIVEN the <EVENT>
    WHEN the <CAUSE>
    THEN the <ACTION>

---

### Low Battery Alarm (10 minutes) :x:

RiskID | Domain | Functional Area | Standard Reference | Assignee
------ | ------ | --------------- | ------------------ | --------
SW8 | Software-GUI | Alarms | ISO80601-2-12:2020 | Tim Powell

#### Scenario: There is only 10 minutes of battery life left

    GIVEN the alarm payload comes in
    WHEN the alarm is about the battery
    THEN check if the low battery alarm signal is sent
    AND THEN the low battery alarm is displayed

---

### High Pressure Alarm to be HIGH Priority :large_orange_diamond:

RiskID | Domain | Functional Area | Standard Reference | Assignee
------ | ------ | --------------- | ------------------ | --------
SW11 | Software-GUI | Alarms | ISO80601-2-12:2020 | Tim Powell

#### Scenario: Excessive airway pressure applied

    GIVEN the alarm payload received
    WHEN the alarm_code: 7
    THEN the high priority alarm signal should be sent
    AND THEN alarm popup is triggered
    AND THEN alarm is added to alarm list
